@@ -598,7 +598,9 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup (new_value);
+		entry->type = g_strdup ("string");
 	}
 
 	write_config_file ();
@@ -642,7 +644,9 @@ gda_config_set_int (const gchar *path, gint new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup_printf ("%d", new_value);
+		entry->type = g_strdup ("long");
 	}
 
 	write_config_file ();
@@ -686,7 +690,9 @@ gda_config_set_float (const gchar * path, gdouble new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup_printf ("%f", new_value);
+		entry->type = g_strdup ("float");
 	}
 
 	write_config_file ();
@@ -730,7 +736,9 @@ gda_config_set_boolean (const gchar *path, gboolean new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup_printf ("%d", new_value);
+		entry->type = g_strdup ("bool");
 	}
 
 	write_config_file ();
@@ -1126,7 +1134,7 @@ gda_config_free_provider_list (GList *list)
 		GdaProviderInfo *provider_info = (GdaProviderInfo *) l->data;
 
 		if (provider_info != NULL)
-			gda_config_free_provider_info (provider_info);
+			gda_provider_info_free (provider_info);
 	}
 
 	g_list_free (list);
@@ -1136,7 +1144,7 @@ gda_config_free_provider_list (GList *list)
  * gda_config_get_provider_by_name
  * @name: name of the provider to search for.
  *
- * Gets a #GdaProviderInfo structure from the provider list givin its name.
+ * Gets a #GdaProviderInfo structure from the provider list given its name.
  *
  * Returns: a #GdaProviderInfo structure, if found, or %NULL if not found.
  */
@@ -1244,13 +1252,13 @@ gda_provider_info_copy (GdaProviderInfo *src)
 }
 
 /**
- * gda_config_free_provider_info
+ * gda_provider_info_free
  * @provider_info: provider information to free.
  *
  * Deallocates all memory associated to the given #GdaProviderInfo.
  */
 void
-gda_config_free_provider_info (GdaProviderInfo *provider_info)
+gda_provider_info_free (GdaProviderInfo *provider_info)
 {
 	g_return_if_fail (provider_info != NULL);
 
@@ -1268,7 +1276,7 @@ gda_config_free_provider_info (GdaProviderInfo *provider_info)
  *
  * Returns a list of all data sources currently configured in the system.
  * Each of the nodes in the returned GList is a #GdaDataSourceInfo.
- * To free the returned list, call the #gda_config_free_data_source_info
+ * To free the returned list, call the #gda_config_free_data_source_list
  * function.
  *
  * Returns: a GList of #GdaDataSourceInfo structures.
@@ -1325,7 +1333,7 @@ gda_config_get_data_source_list (void)
  * gda_config_find_data_source
  * @name: name of the data source to search for.
  *
- * Gets a #GdaDataSourceInfo structure from the data source list givin its 
+ * Gets a #GdaDataSourceInfo structure from the data source list given its 
  * name.
  *
  * Returns: a #GdaDataSourceInfo structure, if found, or %NULL if not found.
@@ -1652,7 +1660,7 @@ gda_config_add_listener (const gchar *path,
  * @id: the ID of the listener to remove.
  *
  * Removes a configuration listener previously installed with
- * #gda_config_add_listener, givin its ID.
+ * #gda_config_add_listener, given its ID.
  */
 void
 gda_config_remove_listener (guint id)
@@ -1679,7 +1687,7 @@ gda_provider_info_get_type (void)
 	if (our_type == 0)
 		our_type = g_boxed_type_register_static ("GdaProviderInfo",
 			(GBoxedCopyFunc) gda_provider_info_copy,
-			(GBoxedFreeFunc) gda_config_free_provider_info);
+			(GBoxedFreeFunc) gda_provider_info_free);
 
   return our_type;
 }
