@@ -25,6 +25,7 @@
 #include <libgda/gda-intl.h>
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-log.h>
+#include <libgda/gda-util.h>
 #include <libgda/gda-xml-database.h>
 #include <string.h>
 
@@ -37,8 +38,8 @@ struct _GdaDataModelPrivate {
 	gchar *cmd_text;
 	GdaCommandType cmd_type;
 
-	/* edition mode */
-	gboolean editing;
+	/* update mode */
+	gboolean updating;
 };
 
 static void gda_data_model_class_init (GdaDataModelClass *klass);
@@ -50,9 +51,12 @@ enum {
 	ROW_INSERTED,
 	ROW_UPDATED,
 	ROW_REMOVED,
-	BEGIN_EDIT,
-	CANCEL_EDIT,
-	END_EDIT,
+	COLUMN_INSERTED,
+	COLUMN_UPDATED,
+	COLUMN_REMOVED,
+	BEGIN_UPDATE,
+	CANCEL_UPDATE,
+	END_UPDATE,
 	LAST_SIGNAL
 };
 
@@ -102,45 +106,72 @@ gda_data_model_class_init (GdaDataModelClass *klass)
                               NULL, NULL,
                               g_cclosure_marshal_VOID__INT,
                               G_TYPE_NONE, 1, G_TYPE_INT);
-	gda_data_model_signals[BEGIN_EDIT] =
-		g_signal_new ("begin_edit",
+	gda_data_model_signals[COLUMN_INSERTED] =
+		g_signal_new ("column_inserted",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, begin_edit),
+                              G_STRUCT_OFFSET (GdaDataModelClass, column_inserted),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__INT,
+                              G_TYPE_NONE, 1, G_TYPE_INT);
+	gda_data_model_signals[COLUMN_UPDATED] =
+		g_signal_new ("column_updated",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaDataModelClass, column_updated),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__INT,
+                              G_TYPE_NONE, 1, G_TYPE_INT);
+	gda_data_model_signals[COLUMN_REMOVED] =
+		g_signal_new ("column_removed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaDataModelClass, column_removed),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__INT,
+                              G_TYPE_NONE, 1, G_TYPE_INT);
+	gda_data_model_signals[BEGIN_UPDATE] =
+		g_signal_new ("begin_update",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaDataModelClass, begin_update),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE, 0);
-	gda_data_model_signals[CANCEL_EDIT] =
-		g_signal_new ("cancel_edit",
+	gda_data_model_signals[CANCEL_UPDATE] =
+		g_signal_new ("cancel_update",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, cancel_edit),
+                              G_STRUCT_OFFSET (GdaDataModelClass, cancel_update),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE, 0);
-	gda_data_model_signals[END_EDIT] =
-		g_signal_new ("end_edit",
+	gda_data_model_signals[END_UPDATE] =
+		g_signal_new ("end_update",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, end_edit),
+                              G_STRUCT_OFFSET (GdaDataModelClass, end_update),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE, 0);
 
 	object_class->finalize = gda_data_model_finalize;
 	klass->changed = NULL;
-	klass->begin_edit = NULL;
-	klass->cancel_edit = NULL;
-	klass->end_edit = NULL;
+	klass->begin_update = NULL;
+	klass->cancel_update = NULL;
+	klass->end_update = NULL;
 	klass->get_n_rows = NULL;
 	klass->get_n_columns = NULL;
 	klass->describe_column = NULL;
 	klass->get_row = NULL;
 	klass->get_value_at = NULL;
-	klass->is_editable = NULL;
+	klass->is_updatable = NULL;
 	klass->append_row = NULL;
 	klass->remove_row = NULL;
 	klass->update_row = NULL;
+	klass->append_column = NULL;
+	klass->remove_column = NULL;
+	klass->update_column = NULL;
 }
 
 static void
@@ -152,7 +183,7 @@ gda_data_model_init (GdaDataModel *model, GdaDataModelClass *klass)
 	model->priv->notify_changes = TRUE;
 	model->priv->column_titles = g_hash_table_new (g_direct_hash,
 						       g_direct_equal);
-	model->priv->editing = FALSE;
+	model->priv->updating = FALSE;
 	model->priv->cmd_text = NULL;
 	model->priv->cmd_type = GDA_COMMAND_TYPE_INVALID;
 }
@@ -202,7 +233,7 @@ gda_data_model_get_type (void)
 				0,
 				(GInstanceInitFunc) gda_data_model_init
 			};
-			type = g_type_register_static (PARENT_TYPE, "GdaDataModel", &info, 0);
+			type = g_type_register_static (PARENT_TYPE, "GdaDataModel", &info, G_TYPE_FLAG_ABSTRACT);
 		}
 	}
 
@@ -213,7 +244,7 @@ gda_data_model_get_type (void)
  * gda_data_model_changed
  * @model: a #GdaDataModel object.
  *
- * Notify listeners of the given data model object of changes
+ * Notifies listeners of the given data model object of changes
  * in the underlying data. Listeners usually will connect
  * themselves to the "changed" signal in the #GdaDataModel
  * class, thus being notified of any new data being appended
@@ -233,7 +264,10 @@ gda_data_model_changed (GdaDataModel *model)
 
 /**
  * gda_data_model_row_inserted
- * model: a #GdaDataModel object.
+ * @model: a #GdaDataModel object.
+ * @row: row number.
+ *
+ * Emits the 'row_inserted' and 'changed' signals on @model.
  */
 void
 gda_data_model_row_inserted (GdaDataModel *model, gint row)
@@ -244,12 +278,17 @@ gda_data_model_row_inserted (GdaDataModel *model, gint row)
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[ROW_INSERTED],
 			       0, row);
+
+		gda_data_model_changed (model);
 	}
 }
 
 /**
  * gda_data_model_row_updated
- * model: a #GdaDataModel object.
+ * @model: a #GdaDataModel object.
+ * @row: row number.
+ *
+ * Emits the 'row_updated' and 'changed' signals on @model.
  */
 void
 gda_data_model_row_updated (GdaDataModel *model, gint row)
@@ -260,12 +299,17 @@ gda_data_model_row_updated (GdaDataModel *model, gint row)
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[ROW_UPDATED],
 			       0, row);
+
+		gda_data_model_changed (model);
 	}
 }
 
 /**
  * gda_data_model_row_removed
- * model: a #GdaDataModel object.
+ * @model: a #GdaDataModel object.
+ * @row: row number.
+ *
+ * Emits the 'row_removed' and 'changed' signal on @model.
  */
 void
 gda_data_model_row_removed (GdaDataModel *model, gint row)
@@ -276,6 +320,71 @@ gda_data_model_row_removed (GdaDataModel *model, gint row)
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[ROW_REMOVED],
 			       0, row);
+
+		gda_data_model_changed (model);
+	}
+}
+
+/**
+ * gda_data_model_column_inserted
+ * @model: a #GdaDataModel object.
+ * @col: column number.
+ *
+ * Emits the 'column_inserted' and 'changed' signals on @model.
+ */
+void
+gda_data_model_column_inserted (GdaDataModel *model, gint col)
+{
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+
+	if (model->priv->notify_changes) {
+		g_signal_emit (G_OBJECT (model),
+			       gda_data_model_signals[COLUMN_INSERTED],
+			       0, col);
+
+		gda_data_model_changed (model);
+	}
+}
+
+/**
+ * gda_data_model_column_updated
+ * @model: a #GdaDataModel object.
+ * @col: column number.
+ *
+ * Emits the 'column_updated' and 'changed' signals on @model.
+ */
+void
+gda_data_model_column_updated (GdaDataModel *model, gint col)
+{
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+
+	if (model->priv->notify_changes) {
+		g_signal_emit (G_OBJECT (model),
+			       gda_data_model_signals[COLUMN_UPDATED],
+			       0, col);
+
+		gda_data_model_changed (model);
+	}
+}
+
+/**
+ * gda_data_model_column_removed
+ * @model: a #GdaDataModel object.
+ * @col: column number.
+ *
+ * Emits the 'column_removed' and 'changed' signal on @model.
+ */
+void
+gda_data_model_column_removed (GdaDataModel *model, gint col)
+{
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+
+	if (model->priv->notify_changes) {
+		g_signal_emit (G_OBJECT (model),
+			       gda_data_model_signals[COLUMN_REMOVED],
+			       0, col);
+
+		gda_data_model_changed (model);
 	}
 }
 
@@ -283,7 +392,7 @@ gda_data_model_row_removed (GdaDataModel *model, gint row)
  * gda_data_model_freeze
  * @model: a #GdaDataModel object.
  *
- * Disable notifications of changes on the given data model. To
+ * Disables notifications of changes on the given data model. To
  * re-enable notifications again, you should call the
  * #gda_data_model_thaw function.
  */
@@ -298,7 +407,7 @@ gda_data_model_freeze (GdaDataModel *model)
  * gda_data_model_thaw
  * @model: a #GdaDataModel object.
  *
- * Re-enable notifications of changes on the given data model.
+ * Re-enables notifications of changes on the given data model.
  */
 void
 gda_data_model_thaw (GdaDataModel *model)
@@ -311,7 +420,7 @@ gda_data_model_thaw (GdaDataModel *model)
  * gda_data_model_get_n_rows
  * @model: a #GdaDataModel object.
  *
- * Return the number of rows in the given data model.
+ * Returns: the number of rows in the given data model.
  */
 gint
 gda_data_model_get_n_rows (GdaDataModel *model)
@@ -326,7 +435,7 @@ gda_data_model_get_n_rows (GdaDataModel *model)
  * gda_data_model_get_n_columns
  * @model: a #GdaDataModel object.
  *
- * Return the number of columns in the given data model.
+ * Returns: the number of columns in the given data model.
  */
 gint
 gda_data_model_get_n_columns (GdaDataModel *model)
@@ -342,7 +451,7 @@ gda_data_model_get_n_columns (GdaDataModel *model)
  * @model: a #GdaDataModel object.
  * @col: column number.
  *
- * Query the underlying data model implementation for a description
+ * Queries the underlying data model implementation for a description
  * of a given column. That description is returned in the form of
  * a #GdaFieldAttributes structure, which contains all the information
  * about the given column in the data model.
@@ -384,7 +493,7 @@ gda_data_model_describe_column (GdaDataModel *model, gint col)
  * @model: a #GdaDataModel object.
  * @col: column number.
  *
- * Return the title for the given column in a data model object.
+ * Returns: the title for the given column in a data model object.
  */
 const gchar *
 gda_data_model_get_column_title (GdaDataModel *model, gint col)
@@ -424,6 +533,8 @@ gda_data_model_get_column_title (GdaDataModel *model, gint col)
  * @model: a #GdaDataModel object.
  * @col: column number
  * @title: title for the given column.
+ *
+ * Sets the @title of the given @col in @model.
  */
 void
 gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *title)
@@ -454,7 +565,7 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
  * @model: a #GdaDataModel object.
  * @title: column title.
  *
- * Get the position of a column on the data model, based on
+ * Gets the position of a column on the data model, based on
  * the column's title.
  *
  * Returns: the position of the column in the data model, or -1
@@ -487,7 +598,7 @@ gda_data_model_get_column_position (GdaDataModel *model, const gchar *title)
  * @model: a #GdaDataModel object.
  * @row: row number.
  *
- * Retrieve a given row from a data model.
+ * Retrieves a given row from a data model.
  *
  * Returns: a #GdaRow object.
  */
@@ -506,13 +617,13 @@ gda_data_model_get_row (GdaDataModel *model, gint row)
  * @col: column number.
  * @row: row number.
  *
- * Retrieve the data stored in the given position (identified by
+ * Retrieves the data stored in the given position (identified by
  * the @col and @row parameters) on a data model.
  *
  * This is the main function for accessing data in a model.
  *
  * Returns: a #GdaValue containing the value stored in the given
- * position, or NULL on error (out-of-bound position, etc).
+ * position, or %NULL on error (out-of-bound position, etc).
  */
 const GdaValue *
 gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row)
@@ -524,38 +635,44 @@ gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row)
 }
 
 /**
- * gda_data_model_is_editable
+ * gda_data_model_is_updatable
  * @model: a #GdaDataModel object.
  *
- * Check whether the given data model can be edited or not.
+ * Checks whether the given data model can be updated or not.
  *
- * Returns: TRUE if it can be edited, FALSE if not.
+ * Returns: %TRUE if it can be updated, %FALSE if not.
  */
 gboolean
-gda_data_model_is_editable (GdaDataModel *model)
+gda_data_model_is_updatable (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (CLASS (model)->is_editable != NULL, FALSE);
+	g_return_val_if_fail (CLASS (model)->is_updatable != NULL, FALSE);
 
-	return CLASS (model)->is_editable (model);
+	return CLASS (model)->is_updatable (model);
 }
 
 /**
  * gda_data_model_append_row
  * @model: a #GdaDataModel object.
- * @values: the row to add.
+ * @values: #GList of #GdaValue* representing the row to add.  The
+ *          length must match model's column count.  These #GdaValue
+ *	    are value-copied.  The user is still responsible for freeing them.
  *
- * Append a row to the given data model.
+ * Appends a row to the given data model.
  *
- * Returns: the unique ID of the added row.
+ * Returns: the added row.
  */
 const GdaRow *
 gda_data_model_append_row (GdaDataModel *model, const GList *values)
 {
+	const GdaRow *row;
+
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 	g_return_val_if_fail (CLASS (model)->append_row != NULL, NULL);
 
-	return CLASS (model)->append_row (model, values);
+	row = CLASS (model)->append_row (model, values);
+	gda_data_model_row_inserted (model, gda_row_get_number ((GdaRow *) row));
+	return row;
 }
 
 /**
@@ -563,19 +680,26 @@ gda_data_model_append_row (GdaDataModel *model, const GList *values)
  * @model: a #GdaDataModel object.
  * @row: the #GdaRow to be removed.
  *
- * Remove a row from the data model. This results in the underlying
+ * Removes a row from the data model. This results in the underlying
  * database row being removed in the database.
  *
- * Returns: TRUE if successful, FALSE otherwise.
+ * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
 gda_data_model_remove_row (GdaDataModel *model, const GdaRow *row)
 {
+	gboolean result;
+
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 	g_return_val_if_fail (row != NULL, FALSE);
 	g_return_val_if_fail (CLASS (model)->remove_row != NULL, FALSE);
 
-	return CLASS (model)->remove_row (model, row);
+	result = CLASS (model)->remove_row (model, row);
+	if (result) {
+		gda_data_model_row_removed (model, gda_row_get_number ((GdaRow *) row));
+	}
+
+	return result;
 }
 
 /**
@@ -583,19 +707,107 @@ gda_data_model_remove_row (GdaDataModel *model, const GdaRow *row)
  * @model: a #GdaDataModel object.
  * @row: the #GdaRow to be updated.
  *
- * Update a row data model. This results in the underlying
+ * Updates a row data model. This results in the underlying
  * database row's values being changed.
  *
- * Returns: TRUE if successful, FALSE otherwise.
+ * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
 gda_data_model_update_row (GdaDataModel *model, const GdaRow *row)
 {
+	gboolean result;
+
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 	g_return_val_if_fail (row != NULL, FALSE);
 	g_return_val_if_fail (CLASS (model)->update_row != NULL, FALSE);
 
-	return CLASS (model)->update_row (model, row);
+	result = CLASS (model)->update_row (model, row);
+	if (result) {
+		gda_data_model_row_updated (model, gda_row_get_number ((GdaRow *) row));
+	}
+	return result;
+}
+
+/**
+ * gda_data_model_append_column
+ * @model: a #GdaDataModel object.
+ * @col: a #GdaFieldAttributes describing the column to add.
+ *
+ * Appends a column to the given data model.  If successful, the position of
+ * the new column in the data model is set on @col, and you can grab it using
+ * @gda_field_attributes_get_position.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
+ */
+gboolean
+gda_data_model_append_column (GdaDataModel *model, GdaFieldAttributes *col)
+{
+	gboolean result;
+	
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (CLASS (model)->append_column != NULL, FALSE);
+	g_return_val_if_fail (col != NULL, FALSE);
+
+	result = CLASS (model)->append_column (model, col);
+	if (result) {
+		gda_data_model_column_inserted (model,
+						gda_field_attributes_get_position (col));
+	}
+
+	return result;
+}
+
+/**
+ * gda_data_model_remove_column
+ * @model: a #GdaDataModel object.
+ * @col: the column to be removed.
+ *
+ * Removes a column from the data model. This means that all values attached to this
+ * column in the data model will be destroyed in the underlying database.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
+ */
+gboolean
+gda_data_model_remove_column (GdaDataModel *model, const GdaFieldAttributes *col)
+{
+	gboolean result;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (CLASS (model)->remove_column != NULL, FALSE);
+	g_return_val_if_fail (col != NULL, FALSE);
+
+	result = CLASS (model)->remove_column (model, col);
+	if (result) {
+		gda_data_model_column_removed (model, gda_field_attributes_get_position (col));
+	}
+
+	return result;
+}
+
+/**
+ * gda_data_model_update_column
+ * @model: a #GdaDataModel object.
+ * @col: the column to be updated.
+ *
+ * Updates a column in the given data model. This results in the underlying
+ * database row's values being changed.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
+ */
+gboolean
+gda_data_model_update_column (GdaDataModel *model, const GdaFieldAttributes *col)
+{
+	gboolean result;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (CLASS (model)->update_row != NULL, FALSE);
+	g_return_val_if_fail (col != NULL, FALSE);
+
+	result = CLASS (model)->update_column (model, col);
+	if (result) {
+		gda_data_model_column_updated (model, gda_field_attributes_get_position (col));
+	}
+	return result;
 }
 
 /**
@@ -604,7 +816,7 @@ gda_data_model_update_row (GdaDataModel *model, const GdaRow *row)
  * @func: callback function.
  * @user_data: context data for the callback function.
  *
- * Call the specified callback function for each row in the data model.
+ * Calls the specified callback function for each row in the data model.
  * This will just traverse all rows, and call the given callback
  * function for each of them.
  *
@@ -614,123 +826,116 @@ gda_data_model_update_row (GdaDataModel *model, const GdaRow *row)
  *
  * where "row" would be the row being read, and "user_data" the parameter
  * specified in @user_data in the call to gda_data_model_foreach.
- * This callback function can return FALSE to stop the processing. If it
- * returns TRUE, processing will continue until no rows remain.
+ * This callback function can return %FALSE to stop the processing. If it
+ * returns %TRUE, processing will continue until no rows remain.
  */
 void
 gda_data_model_foreach (GdaDataModel *model,
 			GdaDataModelForeachFunc func,
 			gpointer user_data)
 {
-	gint cols;
 	gint rows;
-	gint c, r;
-	GdaRow *row;
+	gint r;
+	const GdaRow *row;
+	gboolean more;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	g_return_if_fail (func != NULL);
 
 	rows = gda_data_model_get_n_rows (model);
-	cols = gda_data_model_get_n_columns (model);
-
-	for (r = 0; r < rows; r++) {
-		row = gda_row_new (model, cols);
-		for (c = 0; c < cols; c++) {
-			GdaValue *value;
-			value = gda_value_copy (gda_data_model_get_value_at (model, c, r));
-			memcpy (gda_row_get_value (row, c), value, sizeof (GdaValue));
-		}
-
+	more = TRUE;
+	for (r = 0; more && r < rows ; r++) {
+		row = gda_data_model_get_row (model, r);
 		/* call the callback function */
-		func (model, row, user_data);
-
-		gda_row_free (row);
+		more = func (model, (GdaRow *) row, user_data);
 	}
 }
 
 /**
- * gda_data_model_is_editing
+ * gda_data_model_has_changed
  * @model: a #GdaDataModel object.
  *
- * Check whether this data model is in editing mode or not. Editing
- * mode is set to TRUE when @gda_data_model_begin_edit has been
- * called successfully, and is not set back to FALSE until either
- * @gda_data_model_cancel_edit or @gda_data_model_end_edit have
+ * Checks whether this data model is in updating mode or not. Updating
+ * mode is set to %TRUE when @gda_data_model_begin_update has been
+ * called successfully, and is not set back to %FALSE until either
+ * @gda_data_model_cancel_update or @gda_data_model_end_update have
  * been called.
  *
- * Returns: TRUE if editing mode, FALSE otherwise.
+ * Returns: %TRUE if updating mode, %FALSE otherwise.
  */
 gboolean
-gda_data_model_is_editing (GdaDataModel *model)
+gda_data_model_has_changed (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	return model->priv->editing;
+	return model->priv->updating;
 }
 
 /**
- * gda_data_model_begin_edit
+ * gda_data_model_begin_update
  * @model: a #GdaDataModel object.
  *
- * Start edition of this data model. This function should be the
+ * Starts update of this data model. This function should be the
  * first called when modifying the data model.
  *
- * Returns: TRUE on success, FALSE if there was an error.
+ * Returns: %TRUE on success, %FALSE if there was an error.
  */
 gboolean
-gda_data_model_begin_edit (GdaDataModel *model)
+gda_data_model_begin_update (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (model->priv->editing == FALSE, FALSE);
+	g_return_val_if_fail (model->priv->updating == FALSE, FALSE);
 
-	if (!gda_data_model_is_editable (model)) {
-		gda_log_error (_("Data model %p is not editable"), model);
+	if (!gda_data_model_is_updatable (model)) {
+		gda_log_error (_("Data model %p is not updatable"), model);
 		return FALSE;
 	}
 
-	model->priv->editing = TRUE;
-	g_signal_emit (G_OBJECT (model), gda_data_model_signals[BEGIN_EDIT], 0);
+	model->priv->updating = TRUE;
+	g_signal_emit (G_OBJECT (model),
+		       gda_data_model_signals[BEGIN_UPDATE], 0);
 
-	return model->priv->editing;
+	return model->priv->updating;
 }
 
 /**
- * gda_data_model_cancel_edit
+ * gda_data_model_cancel_update
  * @model: a #GdaDataModel object.
  *
- * Cancels edition of this data model. This means that all changes
+ * Cancels update of this data model. This means that all changes
  * will be discarded, and the old data put back in the model.
  *
- * Returns: TRUE on success, FALSE if there was an error.
+ * Returns: %TRUE on success, %FALSE if there was an error.
  */
 gboolean
-gda_data_model_cancel_edit (GdaDataModel *model)
+gda_data_model_cancel_update (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (model->priv->editing, FALSE);
+	g_return_val_if_fail (model->priv->updating, FALSE);
 
-	g_signal_emit (G_OBJECT (model), gda_data_model_signals[CANCEL_EDIT], 0);
-	model->priv->editing = FALSE;
+	g_signal_emit (G_OBJECT (model),
+		       gda_data_model_signals[CANCEL_UPDATE], 0);
+	model->priv->updating = FALSE;
 
 	return TRUE;
 }
 
 /**
- * gda_data_model_end_edit
+ * gda_data_model_end_update
  * @model: a #GdaDataModel object.
  *
  * Approves all modifications and send them to the underlying
  * data source/store.
  *
- * Returns: TRUE on success, FALSE if there was an error.
+ * Returns: %TRUE on success, %FALSE if there was an error.
  */
 gboolean
-gda_data_model_end_edit (GdaDataModel *model)
+gda_data_model_end_update (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (model->priv->editing, FALSE);
+	g_return_val_if_fail (model->priv->updating, FALSE);
 
-	g_signal_emit (G_OBJECT (model), gda_data_model_signals[END_EDIT], 0);
-	model->priv->editing = FALSE;
+	g_signal_emit (G_OBJECT (model), gda_data_model_signals[END_UPDATE], 0);
+	model->priv->updating = FALSE;
 
 	return TRUE;
 }
@@ -778,7 +983,7 @@ export_to_separated (GdaDataModel *model, gchar sep)
  * gda_data_model_to_comma_separated
  * @model: a #GdaDataModel object.
  *
- * Convert the given model into a comma-separated series of rows.
+ * Converts the given model into a comma-separated series of rows.
  *
  * Returns: the representation of the model. You should free this
  * string when you no longer need it.
@@ -791,6 +996,12 @@ gda_data_model_to_comma_separated (GdaDataModel *model)
 
 /**
  * gda_data_model_to_tab_separated
+ * @model: a #GdaDataModel object.
+ *
+ * Converts the given model into a tab-separated series of rows.
+ *
+ * Returns: the representation of the model. You should free this
+ * string when you no longer need it.
  */
 gchar *
 gda_data_model_to_tab_separated (GdaDataModel *model)
@@ -800,21 +1011,32 @@ gda_data_model_to_tab_separated (GdaDataModel *model)
 
 /**
  * gda_data_model_to_xml
+ * @model: a #GdaDataModel object.
+ * @standalone: whether ... 
+ *
+ * Converts the given model into a XML representation.
+ *
+ * Returns: the representation of the model. You should free this
+ * string when you no longer need it.
  */
 gchar *
 gda_data_model_to_xml (GdaDataModel *model, gboolean standalone)
 {
-	xmlDocPtr xml_doc;
-	xmlNodePtr xml_root, xml_node, xml_tables;
-	gchar *xml = NULL;
 	xmlChar *xml_contents;
+	xmlNodePtr xml_node;
+	gchar *xml;
 	gint size;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
 	xml_node = gda_data_model_to_xml_node (model, "exported_model");
+
 	if (standalone) {
+		xmlDocPtr xml_doc;
+		xmlNodePtr xml_root, xml_tables;
+	
 		xml_doc = xmlNewDoc ("1.0");
+		
 		xml_root = xmlNewDocNode (xml_doc, NULL, "database", NULL);
 		xmlDocSetRootElement (xml_doc, xml_root);
 
@@ -823,7 +1045,15 @@ gda_data_model_to_xml (GdaDataModel *model, gboolean standalone)
 
 		xmlDocDumpMemory (xml_doc, &xml_contents, &size);
 		xmlFreeDoc (xml_doc);
-
+		
+		xml = g_strdup (xml_contents);
+		xmlFree (xml_contents);
+	}
+	else {
+		xmlBufferPtr xml_buf = xmlBufferCreate ();
+		xmlNodeDump (xml_buf, NULL, xml_node, 0, 0); 
+		xml_contents = (xmlChar *) xmlBufferContent (xml_buf);
+		xmlBufferFree (xml_buf);
 		xml = g_strdup (xml_contents);
 	}
 
@@ -847,17 +1077,17 @@ xml_set_int (xmlNodePtr node, const gchar *name, gint value)
 
 /**
  * gda_data_model_to_xml_node
- * @model: A #GdaDataModel object.
- * @name: Name to use for the XML resulting table.
+ * @model: a #GdaDataModel object.
+ * @name: name to use for the XML resulting table.
  *
- * Convert a #GdaDataModel into a xmlNodePtr (as used in libxml).
+ * Converts a #GdaDataModel into a xmlNodePtr (as used in libxml).
  *
  * Returns: a xmlNodePtr representing the whole data model.
  */
 xmlNodePtr
 gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 {
-	xmlNodePtr *node;
+	xmlNodePtr node;
 	gint cols, rows, i;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
@@ -908,8 +1138,9 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 
 				value = gda_data_model_get_value_at (model, c, r);
 				str = gda_value_stringify (value);
-				field = xmlNewChild (row, NULL, "field", str);
+				field = xmlNewChild (row, NULL, "value", str);
 				xml_set_int (field, "position", c);
+				xmlSetProp (field, "gdatype", gda_type_to_string (gda_value_get_type (value)));
 
 				g_free (str);
 			}
@@ -919,11 +1150,108 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 	return node;
 }
 
+static gboolean
+add_xml_row (GdaDataModel *model, xmlNodePtr xml_row)
+{
+	xmlNodePtr xml_field;
+	GList *value_list = NULL;
+	GPtrArray *values;
+	gint i;
+	gboolean retval = TRUE;
+
+	values = g_ptr_array_new ();
+	g_ptr_array_set_size (values, gda_data_model_get_n_columns (model));
+	for (xml_field = xml_row->xmlChildrenNode; xml_field != NULL; xml_field = xml_field->next) {
+		gint pos;
+		GdaValue *value;
+
+		if (strcmp (xml_field->name, "value"))
+			continue;
+
+		pos = atoi (xmlGetProp (xml_field, "position"));
+		if (pos < 0 || pos >= gda_data_model_get_n_columns (model)) {
+			g_warning ("add_xml_row(): invalid position on 'field' node");
+			retval = FALSE;
+			break;
+		}
+
+		if (g_ptr_array_index (values, pos) != NULL) {
+			g_warning ("add_xml_row(): two fields with the same position");
+			retval = FALSE;
+			break;
+		}
+
+		/* create the value for this field */
+		value = gda_value_new_from_xml ((const xmlNodePtr) xml_field);
+		if (!value) {
+			g_warning ("add_xml_row(): cannot retrieve value from XML node");
+			retval = FALSE;
+			break;
+		}
+
+		g_ptr_array_index (values, pos) = value;
+	}
+
+	if (retval) {
+		for (i = 0; i < values->len; i++) {
+			GdaValue *value = (GdaValue *) g_ptr_array_index (values, i);
+
+			if (!value) {
+				g_warning ("add_xml_row(): there are missing values on the XML node");
+				retval = FALSE;
+				break;
+			}
+
+			value_list = g_list_append (value_list, value);
+		}
+
+		if (retval)
+			gda_data_model_append_row (model, value_list);
+
+		g_list_free (value_list);
+	}
+
+	for (i = 0; i < values->len; i++)
+		gda_value_free ((GdaValue *) g_ptr_array_index (values, i));
+
+	return retval;
+}
+
+/**
+ * gda_data_model_add_data_from_xml_node
+ * @model: a #GdaDataModel.
+ * @node: a XML node representing a <data> XML node.
+ *
+ * Adds the data from a XML node to the given data model.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
+ */
+gboolean
+gda_data_model_add_data_from_xml_node (GdaDataModel *model, xmlNodePtr node)
+{
+	xmlNodePtr children;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (node != NULL, FALSE);
+
+	if (strcmp (node->name, "data") != 0)
+		return FALSE;
+
+	for (children = node->xmlChildrenNode; children != NULL; children = children->next) {
+		if (!strcmp (children->name, "row")) {
+			if (!add_xml_row (model, children))
+				return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 /**
  * gda_data_model_get_command_text
  * @model: a #GdaDataModel.
  *
- * Get the text of command that generated this data model.
+ * Gets the text of command that generated this data model.
  *
  * Returns: a string with the command issued.
  */
@@ -938,6 +1266,8 @@ gda_data_model_get_command_text (GdaDataModel *model)
  * gda_data_model_set_command_text
  * @model: a #GdaDataModel.
  * @txt: the command text.
+ *
+ * Sets the command text of the given @model.
  */
 void
 gda_data_model_set_command_text (GdaDataModel *model, const gchar *txt)
@@ -954,7 +1284,7 @@ gda_data_model_set_command_text (GdaDataModel *model, const gchar *txt)
  * gda_data_model_get_command_type
  * @model: a #GdaDataModel.
  *
- * Get the type of command that generated this data model.
+ * Gets the type of command that generated this data model.
  *
  * Returns: a #GdaCommandType.
  */
@@ -969,6 +1299,8 @@ gda_data_model_get_command_type (GdaDataModel *model)
  * gda_data_model_set_command_type
  * @model: a #GdaDataModel.
  * @type: the type of the command (one of #GdaCommandType)
+ *
+ * Sets the type of command that generated this data model.
  */
 void
 gda_data_model_set_command_type (GdaDataModel *model, GdaCommandType type)
